@@ -309,7 +309,7 @@ else:
 if PY2:
     FullArgSpec = namedtuple(
         "FullArgSpec",
-        "args, varargs, varkw, defaults, " "kwonlyargs, kwonlydefaults, annotations",
+        "args, varargs, varkw, defaults, kwonlyargs, kwonlydefaults, annotations",
     )
 
     def getfullargspec(func):
@@ -392,6 +392,10 @@ else:
         "co_cellvars",
     ]
 
+    if sys.version_info >= (3, 8, 0):
+        # PEP 570 added "positional only arguments"
+        CODE_FIELD_ORDER.insert(1, "co_posonlyargcount")
+
 
 def update_code_location(code, newfile, newlineno):
     """Take a code object and lie shamelessly about where it comes from.
@@ -598,3 +602,15 @@ except Exception:
     # Can't use ImportError, because of e.g. Django config errors
     def bad_django_TestCase(runner):
         return False
+
+
+if PY2:
+    LIST_CODES = ("q", "Q", "O")
+else:
+    LIST_CODES = ("O",)
+
+
+def array_or_list(code, contents):
+    if code in LIST_CODES:
+        return list(contents)
+    return array.array(code, contents)
